/*
** iplog_options.c - iplog command line argument handler.
** Copyright (C) 1999 Odin <odin@numb.org>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License, version 2,
** as published by the Free Software Foundation.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA
*/

#include <config.h>

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <syslog.h>
#include <sys/types.h>

#ifndef HAVE_GETOPT_LONG
#	include <gnu/getopt.h>
#else
#	include <getopt.h>
#endif

#include <iplog.h>
#include <iplog_options.h>
#include <iplog_filter.h>

extern int facility;
extern int priority;
extern u_long flags;

static int get_fac(const u_char *facility);
static int get_pri(const u_char *priority);
static void enable(const u_char *arg, u_long flag);
static void print_help(void);

static const char opts[] = "FILNPRSTUbcdefg:hi:kl:nopr:stu:vxz";

static const struct option longopts[] = {
	{"tcp",					optional_argument,	0, 10},
	{"udp",					optional_argument,	0, 11},
	{"icmp",				optional_argument,	0, 12},
	{"facility",			required_argument,	0, 13},
	{"priority",			required_argument,	0, 14},
	{"kill",				no_argument,		0, 'k'},
	{"user",				required_argument,	0, 'u'},
	{"group",				required_argument,	0, 'g'},
	{"rules",				required_argument,	0, 'r'},
	{"ignore",				no_argument,		0, 'd'},
	{"restart",				no_argument,		0, 'R'},
	{"no-fork",				no_argument,		0, 'o'},
	{"stdout",				no_argument,		0, 'L'},
	{"logfile",				required_argument,	0, 'l'},
	{"fool-nmap",			optional_argument,	0, 'z'},
	{"dns-cache",			optional_argument,	0, 'c'},
	{"interface",			required_argument,	0, 'i'},
	{"get-ident",			optional_argument,	0, 'e'},
	{"tcp-resolve",			optional_argument,	0, 'T'},
	{"udp-resolve",			optional_argument,	0, 'U'},
	{"icmp-resolve",		optional_argument,	0, 'I'},
	{"disable-resolver",	no_argument,		0, 'N'},
	{"detect-smurf",		optional_argument,	0, 'S'},
	{"detect-bogus",		optional_argument,	0, 'b'},
	{"detect-portscan",		optional_argument,	0, 'p'},
	{"detect-udp-scan",		optional_argument,	0, 'F'},
	{"detect-ping-flood",	optional_argument,	0, 'P'},
	{"detect-fin-scan",		optional_argument,	0, 'f'},
	{"detect-xmas-scan",	optional_argument,	0, 'x'},
	{"detect-null-scan",	optional_argument,	0, 'n'},
	{"detect-syn-flood",	optional_argument,	0, 's'},
	{"detect-traceroute",	optional_argument,	0, 't'},
	{"version",				no_argument,		0, 'v'},
	{"help",				no_argument,		0, 'h'},
	{NULL, 0, NULL, 0}
};

void get_options(int argc, char *const argv[]) {
	extern u_char *logfile, *user, *group, *ifstring, *rules;
	int opt;

	while ((opt = getopt_long(argc, argv, opts, longopts, NULL)) != EOF) {
		switch (opt) {	
			case 10:
				enable(optarg, LOG_TCP);
				break;
			case 11:
				enable(optarg, LOG_UDP);
				break;
			case 12:
				enable(optarg, LOG_ICMP);
				break;
			case 13:
				facility = get_fac(optarg);				
				break;
			case 14:
				priority = get_pri(optarg);
				break;
			case 'd':
				flags |= IGNORE_NS;
				break;
			case 'u':
				user = optarg;
				break;
			case 'g':
				group = optarg;
				break;
			case 'k':
				kill_iplog(15);
			case 'l':
				logfile = optarg;
				break;
			case 'L':
				logfile = "stdout";
				flags |= LOG_STDOUT;
				break;
			case 'o':
				flags |= NO_FORK;
				break;
			case 'c':
				enable(optarg, DNS_CACHE);
				break;
			case 'e':
#ifdef __linux__
				enable(optarg, GET_IDENT);
#else
				fprintf(stderr,
					"Warning: ident lookups are only supported on Linux.");
#endif
				break;
			case 'T':
				enable(optarg, TCP_RES);
				break;
			case 'U':
				enable(optarg, UDP_RES);
				break;
			case 'I':
				enable(optarg, ICMP_RES);
				break;
			case 'S':
				enable(optarg, SMURF);
				break;
			case 'b':
				enable(optarg, BOGUS);
				break;
			case 'P':
				enable(optarg, PING_FLOOD);
				break;
			case 'p':
				enable(optarg, PORTSCAN);
				break;
			case 'x':
				enable(optarg, XMAS_SCAN);
				break;
			case 'f':
				enable(optarg, FIN_SCAN);
				break;
			case 'F':
				enable(optarg, UDP_SCAN);
				break;
			case 'N':
				flags |= NO_RESOLV;
				break;
			case 'n':
				enable(optarg, NULL_SCAN);
				break;
			case 's':
				enable(optarg, SYN_FLOOD);
				break;
			case 't':
				enable(optarg, TRACEROUTE);
				break;
			case 'i':
				ifstring = optarg;
				break;
			case 'r':
				rules = optarg;
				break;
			case 'R':
				kill_iplog(1);
				break;
			case 'z':
				enable(optarg, FOOL_NMAP);
				break;
			case 'v':
				puts("iplog version " VERSION " (alpha - 10171999)");
				puts("by " AUTHORS"\n"WEBPAGE);
				exit(0);
			case 'h':
			default:
				print_help();
				break;
		}
	}
}

static void enable(const u_char *arg, u_long flag) {
	if (arg == NULL || !strcasecmp(arg, "yes") || !strcasecmp(arg, "true"))
		flags |= flag;
	else if (!strcasecmp(arg, "no") || !strcasecmp(arg, "false"))
		flags &= ~flag;
	else if (!strcasecmp(arg, "toggle"))
		flags ^= flag;
	else {
		fprintf(stderr,
			"Arguments must be either \"yes\", \"true\", \"no\", \"false\" "
			"or \"toggle\".  You gave \"%s\"\n", optarg);
		print_help();
	}
}

static int get_fac(const u_char *facility) {
	int fac = FACILITY;

	if (!strncasecmp(facility, "log_", 4))
		facility += 4;

	if (!strcasecmp(facility, "auth"))
		fac = LOG_AUTH;
	else if (!strcasecmp(facility, "authpriv"))
		fac = LOG_AUTHPRIV;
	else if (!strcasecmp(facility, "cron"))
		fac = LOG_CRON;
	else if (!strcasecmp(facility, "daemon"))
		fac = LOG_DAEMON;
	else if (!strcasecmp(facility, "ftp"))
		fac = LOG_FTP;
	else if (!strcasecmp(facility, "kern"))
		fac = LOG_KERN;
	else if (!strcasecmp(facility, "lpr"))
		fac = LOG_LPR;
	else if (!strcasecmp(facility, "mail"))
		fac = LOG_MAIL;
	else if (!strcasecmp(facility, "news"))
		fac = LOG_NEWS;
	else if (!strcasecmp(facility, "security"))
		fac = LOG_AUTH;
	else if (!strcasecmp(facility, "syslog"))
		fac = LOG_SYSLOG;
	else if (!strcasecmp(facility, "user"))
		fac = LOG_USER;
	else if (!strcasecmp(facility, "uucp"))
		fac = LOG_UUCP;
	else if (!strcasecmp(facility, "local0"))
		fac = LOG_LOCAL0;
	else if (!strcasecmp(facility, "local1"))
		fac = LOG_LOCAL1;
	else if (!strcasecmp(facility, "local2"))
		fac = LOG_LOCAL2;
	else if (!strcasecmp(facility, "local3"))
		fac = LOG_LOCAL3;
	else if (!strcasecmp(facility, "local4"))
		fac = LOG_LOCAL4;
	else if (!strcasecmp(facility, "local5"))
		fac = LOG_LOCAL5;
	else if (!strcasecmp(facility, "local6"))
		fac = LOG_LOCAL6;
	else if (!strcasecmp(facility, "local7"))
		fac = LOG_LOCAL7;
	else
		printf("Invalid facility: \"%s\" - falling back to default.\n",
				facility);

	return (fac);
}

static int get_pri(const u_char *priority) {
	int pri = PRIORITY;

	if (!strncasecmp(priority, "log_", 4))
		priority += 4;

	if (!strcasecmp(priority, "alert"))
		pri = LOG_ALERT;
	else if (!strcasecmp(priority, "crit"))
		pri = LOG_CRIT;
	else if (!strcasecmp(priority, "debug"))
		pri = LOG_DEBUG;
	else if (!strcasecmp(priority, "emerg"))
		pri = LOG_EMERG;
	else if (!strcasecmp(priority, "err"))
		pri = LOG_ERR;
	else if (!strcasecmp(priority, "error"))
		pri = LOG_ERR;
	else if (!strcasecmp(priority, "info"))
		pri = LOG_INFO;
	else if (!strcasecmp(priority, "notice"))
		pri = LOG_NOTICE;
	else if (!strcasecmp(priority, "panic"))
		pri = LOG_EMERG;
	else if (!strcasecmp(priority, "warn"))
		pri = LOG_WARNING;
	else if (!strcasecmp(priority, "warning"))
		pri = LOG_WARNING;
	else
		printf("Invalid priority: \"%s\" - falling back to default.\n",
				priority);

	return (pri);
}

static void print_help(void) {
	printf(
"Usage: " PACKAGE " [options] (\"*\" Denotes enabled by default)
--user      or -u <user|UID>     Run as user.
--group     or -g <group|GID>    Run with group specified.
--logfile   or -l <file>         Do not use syslog, log to <file>.
--rules     or -r <file>         Use <file> as the filter rules file.
--ignore    or -d                Ignore dns traffic from nameservers listed in
                                 /etc/resolv.conf.
--interface or -i <if0,...,ifN>  Listen on specified interface(s).
--kill      or -k                Kill iplog, if it is running.
--restart   or -R                Restart iplog, if it is running.
--no-fork   or -o                Run in the foreground.
--stdout    or -L                Log to stdout.
--help      or -h                This help screen.
--version   or -v                Print version information and exit.

--facility <facility>            Use the specified syslog facility.
--priority <priority>            Use the specified syslog priority.

--tcp[=true|false|toggle]                      %cLog tcp packets.
--udp[=true|false|toggle]                      %cLog udp packets.
--icmp[=true|false|toggle]                     %cLog icmp packets.

--dns-cache[=true|false|toggle]         or -c  %cUse dns cache.
--get-ident[=true|false|toggle]         or -e  %cGet ident info on connections
                                                to listening ports.

--tcp-resolve[=true|false|toggle]       or -T  %cResolve IPs of tcp data.
--udp-resolve[=true|false|toggle]       or -U  %cResolve IPs of udp data.
--icmp-resolve[=true|false|toggle]      or -I  %cResolve IPs of icmp data.
--disable-resolver                      or -N  %cDo not resolve any IPs.

--fool-nmap[=true|false|toggle]         or -z  %cFool nmap's OS detection.
--detect-traceroute[=true|false|toggle] or -t  %cDetect traceroutes.
--detect-ping-flood[=true|false|toggle] or -P  %cDetect ICMP ping floods.
--detect-smurf[=true|false|toggle]      or -S  %cDetect smurf attacks.
--detect-bogus[=true|false|toggle]      or -b  %cDetect bogus tcp flags.
--detect-portscan[=true|false|toggle]   or -p  %cDetect port scans.
--detect-udp-scan[=true|false|toggle]   or -F  %cDetect UDP scans/floods.
--detect-fin-scan[=true|false|toggle]   or -f  %cDetect FIN scans.
--detect-xmas-scan[=true|false|toggle]  or -x  %cDetect Xmas scans.
--detect-null-scan[=true|false|toggle]  or -n  %cDetect null scans.
--detect-syn-flood[=true|false|toggle]  or -s  %cStop resolving IPs if a
                                                SYN flood is detected.\n",
(flags & LOG_TCP	? '*' : ' '), (flags & LOG_UDP		? '*' : ' '),
(flags & LOG_ICMP	? '*' : ' '), (flags & DNS_CACHE	? '*' : ' '),
(flags & GET_IDENT	? '*' : ' '), (flags & TCP_RES		? '*' : ' '),
(flags & UDP_RES	? '*' : ' '), (flags & ICMP_RES		? '*' : ' '),
(flags & NO_RESOLV	? '*' : ' '), (flags & FOOL_NMAP	? '*' : ' '),
(flags & TRACEROUTE	? '*' : ' '),
(flags & PING_FLOOD	? '*' : ' '), (flags & SMURF		? '*' : ' '),
(flags & BOGUS		? '*' : ' '), (flags & PORTSCAN		? '*' : ' '),
(flags & UDP_SCAN	? '*' : ' '), (flags & FIN_SCAN		? '*' : ' '),
(flags & XMAS_SCAN	? '*' : ' '), (flags & NULL_SCAN	? '*' : ' '),
(flags & SYN_FLOOD	? '*' : ' '));
	exit(0);
}

/* vim:ts=4:sw=8:tw=0 */
